using System.Text;
using Microsoft.AspNetCore.Mvc;
using System.Linq;
using Microsoft.AspNetCore.Http;
using System;
using System.Collections.Generic;
using System.Runtime.Serialization.Formatters.Binary;
using System.IO;
using System.Text.RegularExpressions;

namespace lab10_serverside.Controllers
{
    [Serializable]
    class Table {
	List<List<String>> cells;
	int rowCount;

	public int Rows => rowCount;
	public int Columns => cells.Count;
	
	public void RemoveRow() {
	    foreach(var col in cells) {
		col.RemoveAt(col.Count - 1);
	    }
	    rowCount--;
	}

	public void AddRow() {
	    rowCount++;
	    foreach(var col in cells) {
		col.Add("");
	    }
	}

	public void RemoveCol() {
	    cells.RemoveAt(cells.Count - 1);
	}

	public void AddCol() {
	    cells.Add(Enumerable.Repeat("", rowCount).ToList());
	}

	public String this[int r, int c] {
	    get => cells[c][r];
	    set => cells[c][r]=value;
	}

	public Table(int r, int c) {
	    rowCount = r;
	    cells = new List<List<String>>();
	    for(int i=0; i< c; i++) {
		AddCol();
	    }
	}

	public String Render() {
	    StringBuilder sb = new StringBuilder();

	    sb.Append("<form method=\"post\" action=\"\">");
	    sb.AppendLine("<button type=\"submit\" name=\"action\" value=\"addrow\">Add row</button>");
	    sb.AppendLine("<button type=\"submit\" name=\"action\" value=\"delrow\">Del row</button>");
	    sb.AppendLine("<button type=\"submit\" name=\"action\" value=\"addcol\">Add col</button>");
	    sb.AppendLine("<button type=\"submit\" name=\"action\" value=\"delcol\">Del col</button><br>");
	    sb.AppendLine("<input type=\"submit\"><input type=\"reset\"><table>");
	    for(int r=0;r<rowCount;r++) {
		sb.Append($"<tr>");
		for(int c=0; c < cells.Count; c++) {
		    sb.Append($"<td><input type=\"text\" name=\"r{r}c{c}\" value=\"{this[r,c]}\"></td>");
		}
		sb.AppendLine("</tr>");
	    }
	    sb.AppendLine("</table></form>");

	    return sb.ToString();
	}

	public static byte[] Serialize(Table t) {
	    BinaryFormatter bf = new BinaryFormatter();
	    using(MemoryStream ms = new MemoryStream()) {
		bf.Serialize(ms, t);
		return ms.ToArray();
	    }
	}

	public static Table Deserialize(byte[] inp) {
	    BinaryFormatter bf = new BinaryFormatter();
	    using(MemoryStream ms = new MemoryStream(inp)) {
		return (Table) bf.Deserialize(ms);
	    }
	}
    }
    
    public class Task3Controller : Controller {
	public IActionResult Index() {
	    //Aby sesja działała trzeba dodać AddSession i UseSession w pliku startup
	    byte[] data = HttpContext.Session.Get("table");
	    Table t;
	    if(data != null) {
		t = Table.Deserialize(data);
	    } else {
		t = new Table(3,3);
	    }

	    if(Request.HasFormContentType) {
		switch(Request.Form["action"]) {
		    case "addcol": t.AddCol(); break;
		    case "delcol"  when t.Columns > 1: t.RemoveCol();; break;
			// Silently ignore request to remove last row/column
		    case "addrow": t.AddRow(); break;
		    case "delrow"  when t.Rows > 1: t.RemoveRow();; break;
		}


		Regex regex = new Regex(@"^r(\d+)c(\d+)$");
		foreach(var p in Request.Form) {
		    if(regex.IsMatch(p.Key)) {
			var m = regex.Match(p.Key);
			var row =  int.Parse(m.Groups[1].Value);
			var col =  int.Parse(m.Groups[2].Value);
			if(row < t.Rows && col < t.Columns) 
			    t[row, col] = p.Value;
		    }
		    
		}
	    }

	    HttpContext.Session.Set("table", Table.Serialize(t));

	    
	    StringBuilder sb = new StringBuilder();
	    sb.AppendLine("<!DOCTYPE html>\n<html><head><title>The table</title></head><body>");
	    sb.Append(t.Render());
	    sb.AppendLine("</body></html>");
	    
	    return Content(sb.ToString(), "text/html");
	}
    }
}
